#!/usr/bin/perl
#
# (c) 2000 - Gianluca Pasqualini email: gpasqual@dsi.unive.it
# Universita' degli Studi di Venezia - Dipartimento di Informatica
#
# This script add/remove/show automatically aliases for sympa
# in /etc/aliases.

# This script requires a start/end tag in /etc/aliases to work properly.
# Insert tags in a commented line, 'start' tag for first:
# example:
## --sympa-start--
# ... other uncommented stuff for sympa list aliases goes here
# [...] other stuff
## --sympa-end--
# 
# end tag is optional but is recommended.


# edit these variables if your system don't match
# the same path configuration
my $alias_file = 	 "/etc/aliases";
my $new_alias_file = 	 "/etc/aliases.tmp";
my $newaliases_cmd =	 "/usr/bin/newaliases";
my $sympa_bounce = 	 "/home/sympa/bin/bounce";
my $sympa_bouncequeue =  "/home/sympa/bin/bouncequeue";


my $bounce_traps = ":|-request:|-editor:|-owner:|-subscribe:|-unsubscribe:";

my ( $start_tag, $end_tag ) = ( "--sympa-start--", "--sympa--end--" ); 

sub find_start {
	my ($fh, $nfh) = @_;
	while ( <$fh> ) {
		if ( /$start_tag/ ) {
			print $nfh "# $start_tag\n";
			return;
		} elsif ( /$end_tag/) {
			die "Error: start tag is missing from $alias_file\n";
		} else {
			print $nfh $_;
		}
	}
	die "Error: start tag '--sympa-start--' not found in $alias_file!\n" .
	    "Insert tags to insert aliases automatically.\n";
}

sub already_exists {
	my( $fh, $listname ) = @_;
	my $start_pos = tell( $fh );
	my $found = 0;
	while(<$fh>) {
		if (/$listname($bounce_traps)/) { 
			$found = 1;
			break;
		} elsif (/$end_tag/) {
			break; 
		}
	}
	# place new aliases just after the start tag
	seek( $fh, $start_pos, 0 );
	return $found;
}


sub complete_copy {
	my ($fh, $nfh) = @_;
	while( <$fh> ) { print $nfh $_ ;}
}

if ( $#ARGV != 1 ) { die "Usage: $0 <add|remove|show> <listname>\n";}

my ( $op, $listname ) = @ARGV;

open( ALIAS, "$alias_file" ) or die "Unable to open $alias_file: $!\n";

if ( $op =~ /add/ ) {

	open( NEWALIAS, ">$new_alias_file" ) or die "Unable to create $new_alias_file: $!\n";
	find_start(ALIAS, NEWALIAS);			
	if ( already_exists( ALIAS, $listname )) { die "Error: list $listname already exists in $alias_file\n"; };
	print NEWALIAS "${listname}: \"|$sympa_bounce $listname\"\n";
	print NEWALIAS "${listname}-request: \"|$sympa_bounce ${listname}-request\"\n";
	print NEWALIAS "${listname}-editor: \"|$sympa_bounce ${listname}-editor\"\n";
	print NEWALIAS "${listname}-owner: \"|$sympa_bouncequeue $listname\"\n";
	print NEWALIAS "${listname}-subscribe: \"|$sympa_bounce ${listname}-subscribe\"\n";
	print NEWALIAS "${listname}-unsubscribe: \"|$sympa_bounce ${listname}-unsubscribe\"\n";
	complete_copy(ALIAS, NEWALIAS);
	close NEWALIAS; close ALIAS;
	unlink( $alias_file ) || die "Error: $!\n";
	rename( $new_alias_file, $alias_file ) || die "Error: $!\n";
	system($newaliases_cmd) == 0 ||  print STDERR "couldn't exec $newaliases_cmd: $!"; 
	print "New list $listname added.\n";

} elsif ( $op =~ /remove/ )  {
	my $rows = 0;
	open( NEWALIAS, ">$new_alias_file" ) or die "Unable to create $new_alias_file: $!\n";
	find_start(ALIAS, NEWALIAS);
	while(<ALIAS>) {
		if (/$listname($bounce_traps)/) { 
			print "removing $_";
			$rows++;
			next;
		} elsif (/$end_tag/) { 
			print NEWALIAS $_; 
			break; 
		}
	}
	complete_copy(ALIAS,NEWALIAS);
	close NEWALIAS; close ALIAS;
	unlink( $alias_file ) || die "Error: $!\n";
	rename( $new_alias_file, $alias_file ) || die "Error: $!\n";
	system ($newaliases_cmd) == 0 ||  print STDERR "couldn't exec $newaliases_cmd: $!"; 
	print "$rows rows removed.\n";
	

} elsif ( $op =~ /show/ ) {
	my $rows = 0;
	print "Rows found for list $listname:\n";

	while(<ALIAS>) {
		if ($_ =~ /$listname($bounce_traps)/) { 
			print $_;
			$rows++;
		}
	}
	print "$rows rows found.\n";
	close ALIAS;

} else { die "Invalid command. Use: 'add' 'remove' or 'show'"; }
